using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using C1.Win.C1FlexGrid;

namespace C1FlexCalendarTest
{
    public partial class Form1 : Form
    {
        private ArrayList _selection;

        public Form1()
        {
            InitializeComponent();
        }

        private void monthlyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // switch to monthly view
            monthlyToolStripMenuItem.Checked = true;
            weeklyToolStripMenuItem.Checked = false;
            c1FlexCalendar1.View = C1FlexCalendar.ViewEnum.Monthly;
        }

        private void weeklyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // switch to weekly view
            weeklyToolStripMenuItem.Checked = true;
            monthlyToolStripMenuItem.Checked = false;
            c1FlexCalendar1.View = C1FlexCalendar.ViewEnum.Weekly;
        }

        private void c1FlexCalendar1_CellButtonClick(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            // initiate editing when the user clicks the in-cell button
            if (c1FlexCalendar1.Editor == null)
                c1FlexCalendar1.StartEditing();
        }

        private void c1FlexCalendar1_StartEdit(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            // save the current selection for use in subsequent events
            _selection = c1FlexCalendar1.SelectedTaskList;
        }

        private void c1FlexCalendar1_SetupEditor(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            TextBox tb = c1FlexCalendar1.Editor as TextBox;

            if (tb != null)
            {
                // resize the editor to fit the selected cells
                tb.Size = c1FlexCalendar1.GetSelectionSize();
                
                // enable multiline editing with word wrap
                tb.BackColor = Color.White;
                tb.Multiline = true;
                tb.WordWrap = true;

                // if the selection spans a single appointment, copy the text into the editor
                if (_selection.Count == 1)
                {
                    Appointment app = _selection[0] as Appointment;

                    if (app != null)
                    {
                        tb.Text = app.Task;
                        tb.SelectAll();
                    }
                }
                
                // if the selection spans multiple appointments, clear the editor's text
                else if (_selection.Count > 1)
                {
                    tb.Text = "";
                }
            }
        }

        private void c1FlexCalendar1_AfterEdit(object sender, RowColEventArgs e)
        {
            // get the text the user just typed
            string task = c1FlexCalendar1[e.Row, e.Col].ToString();
            
            // compute the number of days spanned by the current selection
            DateTime start = c1FlexCalendar1.TimeStart;
            DateTime finish = c1FlexCalendar1.TimeEnd;
            int days = finish.Subtract(start).Days;

            // clear the appointments saved previously
            foreach (object o in _selection)
                appointmentsBindingSource.Remove(o);

            // a zero-length string means delete the prior appointments
            if (task.Length > 0)
            {
                for (int i = 0; i <= days; i++)
                {
                    // add a new appointment for each day in the selected range
                    Appointment app = new Appointment();
                    app.Start = start.AddDays(i);
                    app.Finish = finish.AddDays(i - days);
                    app.Task = task;
                    appointmentsBindingSource.Add(app);
                }
            }
           
            // create an ArrayList from the binding source and assign it to the calendar
            ArrayList a = new ArrayList(appointmentsBindingSource.List);
            c1FlexCalendar1.TaskList = a;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            string s = AppDomain.CurrentDomain.BaseDirectory + "C1FlexCalendarTest.xml";

            // copy appointments from the calendar to the serializer
            AppointmentSerializer collection = new AppointmentSerializer();
            collection.Elements = c1FlexCalendar1.TaskList;

            // serialize the appointments to an xml file
            XmlSerializer mySerializer = new XmlSerializer(typeof(AppointmentSerializer));
            StreamWriter myWriter = new StreamWriter(s);
            mySerializer.Serialize(myWriter, collection);
            myWriter.Close();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string s = AppDomain.CurrentDomain.BaseDirectory + "C1FlexCalendarTest.xml";

            // quit if the serialized xml file does not exist
            if (!File.Exists(s))
                return;

            // deserialize appointments from an xml file
            AppointmentSerializer collection = new AppointmentSerializer();
            XmlSerializer mySerializer = new XmlSerializer(typeof(AppointmentSerializer));
            XmlTextReader myReader = new XmlTextReader(s);
            collection = (AppointmentSerializer)mySerializer.Deserialize(myReader);
            myReader.Close();
            
            // copy deserialized appointments to the calendar   
            c1FlexCalendar1.TaskList = collection.Elements;

            // initialize the binding source used for subsequent edits
            foreach (object o in collection.Elements)
                appointmentsBindingSource.Add(o);
         }
    }
}